%% Steps to evaluate optical distortions for grid target.
clc; clear; close all;

%% These steps create a matrix of the distorted postions

%Define image as matrix
Image = imread('grid wd 13mm enhanced0001 cropped.bmp');

% Strip off last layer of array
%Image = Image(:,:,1:3);

%Determine appropriate gray scale threshold to determine what is white and
%what is black.
graylevel = .30;
% graylevel = graythresh(Image);

%Convert image to only black and white (1's and 0's in the matrix) with the
%approprate grayscale theshold
BWImage = im2bw(Image,graylevel);

%Image may need to be inverted if dots are black and bachground is white.
%To invert image uncomment next line
inv_BWImage = ~BWImage;

%fill holes in black/white image
filled=imfill(inv_BWImage,'holes');

%Each reagion must be assigned a different label so the correct number of
%centroids are calculated
labeled_Image = bwlabel(filled,8);

%filter out small areas
A = regionprops(labeled_Image,'Area');
idx = find([A.Area] > 8);
BW_area_filtered = ismember(labeled_Image,idx);

%label image again after filtering
labeled_Image2 = bwlabel(BW_area_filtered,8);

%A structure is created with all of the centroids of the dot grid
struct = regionprops(labeled_Image2,'Centroid');

%Create a matrix with the centroid centers
M_Centroid_distort = cat(1,struct.Centroid);

%% These next steps create a matrix of the orginal positions

% The matrix of the centroid positions needs to be sorted in ascending
% order with respect to the vertical centroid coordinate while maintaining
% the horizontal and vertical correlation
Sorted_M_Centroid_distort = sortrows(M_Centroid_distort,2);

%Length of centroid matrix
Length_M_Centroid_distort = size(Sorted_M_Centroid_distort,1);

%It is assumed there is no distortion at the center of the principle axes.
%This step determines the point all the aberrations are centered around.
aberration_center = round(Length_M_Centroid_distort/2);

%If the actual distance between the centroids is unkown then you need a
%estimate the distance. Estimation is done by taking the average distance
%from the aberration center to the centroids immediately above and below
%it.

% Coordinates of centroids above and below aberration center
coordinate_above = Sorted_M_Centroid_distort(round(Length_M_Centroid_distort/2)-1,2);
coordinate_below = Sorted_M_Centroid_distort(round(Length_M_Centroid_distort/2)+1,2);

Distance = (coordinate_below-coordinate_above)/2;

%Build matrix of original centroid locations. This step only changes the
%vetical location and keeps the horizontal location from the distorted
%image.

%make an empty matrix for filling up
M_Centroid = zeros(Length_M_Centroid_distort,2);

i = 0;
while i < Length_M_Centroid_distort - aberration_center;
    j = i+1;
M_Centroid(aberration_center + j,2) = Sorted_M_Centroid_distort(aberration_center,2) + (Distance * j);
M_Centroid(aberration_center - i,2) = Sorted_M_Centroid_distort(aberration_center,2) - (Distance * i);
M_Centroid(aberration_center + j,1) = Sorted_M_Centroid_distort(aberration_center + i,1);
M_Centroid(aberration_center - i,1) = Sorted_M_Centroid_distort(aberration_center - i,1);
i = i+1;
end

% %% Plot Centroids on orignal image
% imshow(Image)
% hold on
% plot(M_Centroid(:,1), M_Centroid(:,2), 'b*')
% hold off 

%% Plot aberration

% Want to shift plot by coordinate of abberation center
aberration = Sorted_M_Centroid_distort(round(Length_M_Centroid_distort/2),2);

%make plot
plot(M_Centroid(:,2)-aberration,Sorted_M_Centroid_distort(:,2)-aberration, 'bo');

%% Polynomial fitting 
%Choose polynomial order
order = 3;

p = polyfit(M_Centroid(:,2)-aberration,Sorted_M_Centroid_distort(:,2)-aberration,order);
f = polyval(p,M_Centroid(:,2)-aberration);
error=Sorted_M_Centroid_distort(:,2)-aberration-f;


%% Plot fitting results
hold on;
plot(M_Centroid(:,2)-aberration,f,'g', M_Centroid(:,2)-aberration, error,'r')
legend('centroids', 'Y_d_i_s_t_o_r_t_e_d(Y)', 'error')

xlabel('Undistorted Centroid Locations (Y)','FontSize',16)
ylabel('Distorted Centroid Locations (Y_d_i_s_t_o_r_t_e_d)','FontSize',16)